--------------------------------------------------------------------------------
-- |
-- Module      : TreasureHunters.HTML
-- Copyright   : (c) Milan Straka, 2015
-- License     : BSD3
--
-- Maintainer  : fox@ucw.cz
-- Stability   : provisional
-- Portability :
--------------------------------------------------------------------------------

module TreasureHunters.HTML ( htmlPage
                            , htmlRedirect
                            , jsHtmlPage

                            , heading
                            , group
                            , TableAlign(..)
                            , table
                            , monospaced

                            , form
                            , button
                            , input
                            , textArea
                            , radio
                            , href
                            ) where

import TreasureHunters.Server.Resources

htmlPage :: ShowS -> ShowS
htmlPage html = showString headerHTML . showString bodyHTML . html . showString footerHTML

htmlRedirect :: String -> ShowS
htmlRedirect url = showString headerHTML .
                   showString "    <meta http-equiv='refresh' content='0;url=/" . showString url . showString "'>\n" .
                   showString bodyHTML .
                   showString "Pokud jste nebyli přesměrováni, pokračujte " . href url "zde" .
                   showString footerHTML

jsHtmlPage :: ShowS -> ShowS
jsHtmlPage html = showString headerHTML . showString jsHeaderHTML . showString bodyHTML . html . showString footerHTML

panel :: String -> String -> ShowS -> ShowS
panel kind heading content =
  showString "<div class='panel panel-" . showString kind . showString "'><div class='panel-heading'>" .
  showString heading .
  showString "</div>\n<div class='panel-body'>" .
  content .
  showString "</div>\n</div>\n"

heading :: String -> ShowS -> ShowS
heading h1 = panel "primary" $ "<h1 class='text-center'>" ++ h1 ++ "</h1>\n"

group :: String -> ShowS -> ShowS
group = panel "default"

data TableAlign = TableAlignLeft | TableAlignCenter | TableAlignRight deriving Eq

table :: [[(TableAlign, ShowS)]] -> ShowS
table cells = showString "<table style='width: 100%'>" . tableRows False cells . showString "</table>\n"
  where tableRows _ [] = id
        tableRows prev (r:rs) = showString "<tr style='vertical-align: middle'>" . tableCols prev (not $ null rs) False r . showString "</tr>\n" . tableRows True rs
        tableCols _ _ _ [] = id
        tableCols prevRow nextRow prevCol ((a,c):cs) =
          showString ("<td style='text-align:" ++ align ++ padding ++ "'>") . c . showString "</td>" .  tableCols prevRow nextRow True cs
         where align | a == TableAlignLeft = "left"
                     | a == TableAlignCenter = "center"
                     | a == TableAlignRight = "right"
               padding = (if prevRow then "; padding-top: 7.5px" else "") ++
                         (if nextRow then "; padding-bottom: 7.5px" else "") ++
                         (if prevCol then "; padding-left: 7.5px" else "") ++
                         (if not (null cs) then "; padding-right: 7.5px" else "")

monospaced :: ShowS -> ShowS
monospaced content = showString "<div style='font-family: monospace; white-space: pre'>" . content . showString "</div>\n"

form :: String -> ShowS -> ShowS
form url content = showString "<form action='" . showString url . showString "' method='get'>" . content . showString "</form>\n"

button :: String -> ShowS
button label = showString "<button type='submit' class='btn btn-primary form-control'>" . showString label . showString "</button>"

input :: String -> String -> ShowS
input name value = showString "<input type='text' class='form-control' name='" . showString name . showString "' value='" . showString value . showString "'>"

textArea :: String -> Int -> Int -> ShowS -> ShowS
textArea name cols rows value = showString "<textarea cols='" . shows (cols+1) . showString "' rows='" . shows (rows+1) . showString "' name='" . showString name . showString "' style='font-family: monospace' class='form-control'>" . value . showString "</textarea>"

radio :: String -> String -> Bool -> String -> ShowS
radio name value checked label = showString $ "<label class='radio-inline'><input type='radio' name='" ++ name ++
  "' value='" ++ value ++ "'" ++ (if checked then " checked" else "") ++ ">" ++ label ++ "</label>"

href :: String -> String -> ShowS
href url label = showString "<a href='" . showString url . showString "'>" . showString label . showString "</a>"
