{-# LANGUAGE ForeignFunctionInterface, ScopedTypeVariables #-}
--------------------------------------------------------------------------------
-- |
-- Module      : TreasureHunters.Utils
-- Copyright   : (c) Milan Straka, 2015
-- License     : BSD3
--
-- Maintainer  : fox@ucw.cz
-- Stability   : provisional
-- Portability :
--------------------------------------------------------------------------------

module TreasureHunters.Utils ( failure
                             , mapFst
                             , mapSnd
                             , maybeAt
                             , maybeRead
                             , readWithError
                             , readWithFailure
                             , showException
                             , try
                             , verify
                             ) where

import Control.Exception (SomeException, catch)
import Control.Monad
import Prelude hiding (catch)
#ifdef _OS_UNIX
import System.IO (hPutStrLn, stderr)
#elif defined(_OS_WIN32)
import Graphics.Win32 (messageBox, mB_OK, mB_ICONSTOP, nullHANDLE)
#endif

failure :: String -> Either String a
failure = Left

mapFst :: (a -> a') -> (a, b) -> (a', b)
mapFst f (a, b) = (f a, b)

mapSnd :: (b -> b') -> (a, b) -> (a, b')
mapSnd f (a, b) = (a, f b)

maybeAt :: Int -> [a] -> Maybe a
maybeAt _ []     = Nothing
maybeAt 0 (a:_)  = Just a
maybeAt i (_:as) = maybeAt (i-1) as

maybeRead :: Read a => String -> Maybe a
maybeRead str | [(a, "")] <- reads str = Just a
              | otherwise = Nothing

readWithError :: Read a => String -> String -> a
readWithError message str | [(a, "")] <- reads str = a
                          | otherwise = error message

readWithFailure :: Read a => String -> String -> Either String a
readWithFailure message str | [(a, "")] <- reads str = return a
                            | otherwise = failure message

showException :: String -> SomeException -> IO ()
showException msg e = do
  let text = msg ++ ": " ++ (show e)
#ifdef _OS_UNIX
  hPutStrLn stderr text
#elif defined(_OS_WIN32)
  initCommonControls
  void $ messageBox nullHANDLE text "TreasureHunters" (mB_OK + mB_ICONSTOP)

foreign import stdcall unsafe "windows.h InitCommonControls" initCommonControls :: IO ()
#endif

try :: IO a -> a -> IO a
try what err = what `catch` \ (_::SomeException) -> return err

verify :: String -> Bool -> Either String ()
verify reason condition | not condition = failure reason
                        | otherwise = return ()
